#include "multiboot.h"
#include "pic.h"
#include "idt.h"
#include "gdt.h"
#include "task.h"
#include "descgate.h"

/* The following are for the multitasking test
 * I can't use local variables, because they're
 * assigned to the stack in C, and I haven't
 * defined separate stacks for each tasks yet ;)
 *
 * NOTE: Yeah, I've got separate stacks now, so
 * these could be moved into the local address
 * space, but... this is just a test, so I'm
 * not going to bother ;)
 */
char *videoOne = (char *)0x000B8000 + 0xA0;
char *videoTwo = (char *)0x000B8002 + 0xA0;

#define TASK_STACK_SIZE 4096

char *stackOne[TASK_STACK_SIZE];
char *stackTwo[TASK_STACK_SIZE];
Task taskOne, taskTwo, taskMain;
TSS  tssOne,  tssTwo,  tssMain;

void jobOne(void);
void jobTwo(void);

void ndkStart(unsigned long magic, unsigned long addr) {
  /* div by zero test! */
  int x = 3, y = 0;
  /* for the time being, I'll keep this here */
  mb_main(magic, addr);

  idtInit();
  /* setup the PIC */
  picInit();
  timerInit();

  /* Exception handler tests... :)
  //x = x / y;

  // Selector not present... will create an exception...
  createDescriptor(&GDT, 4, 0x0, 0xffffff, D_NOT_PRESENT);
  asm(
    "mov $0x11111111, %eax\n"
    "mov $0x22222222, %ebx\n"
    "mov $0x33333333, %ecx\n"
    "mov $0x44444444, %edx\n"
    "mov $0x12345678, %esi\n"
    "mov $0x87654321, %edi\n"
    "ljmp $0x28, $0x87654321");
  */


  // create a TSS for the main task
  taskFillTSS(&tssMain, 0, 0);
  taskCreate(&taskMain, &tssMain);
  printf("Task Main: 0x%x\n", taskMain.selector);

  // Load TR with the currently executing task (main!)
  printf("Loading TR\n");
  loadTaskReg(taskMain.selector);

  // create a TSS for task one
  taskFillTSS(&tssOne, (long)jobOne, (long)stackOne + TASK_STACK_SIZE);
  taskCreate(&taskOne, &tssOne);
  printf("Task 1: 0x%x\n", taskOne.selector);

  // create a TSS for task two
  taskFillTSS(&tssTwo, (long)jobTwo, (long)stackTwo + TASK_STACK_SIZE);
  taskCreate(&taskTwo, &tssTwo);
  printf("Task 2: 0x%x\n", taskTwo.selector);

  /* by this point, the scheduler is in full effect... no need to
   * tell it to start.  Once it notices two or more tasks in the
   * task queue, it'll start multi-tasking
   */

  asm("stop: \n jmp stop\n");
  return;
}

/* multitasking test!!! */
void jobOne(void) {
  //asm("sti");
  loopOne:
    (*videoOne)++;
    goto loopOne;
}

void jobTwo(void) {
  //asm("sti");
  loopTwo:
    (*videoTwo)++;
    goto loopTwo;
}
